    module issa
    implicit none
    real*8     ::    xmean,xstd
    real*8     ::    ts(:),x(:)
    real*8     ::    lam(:),c(:)
    real*8     ::    a(:,:),t(:,:)
    real*8     ::    u(:,:),vt(:,:)
    real*8     ::    xk(:,:),ro(:,:)
    real*8     ::    rc(:,:),rx(:)
    allocatable::    ts,x,c,lam,a,t,u,vt,&
                     xk,ro,rc,rx
                     
    integer*4,parameter::    n=35040    !length of time series
    integer*4,parameter::    m=120      !window length
    integer*4,parameter::    npc=10     !number of principal component selected
    real*8,parameter   ::    sf=0.5     !percentage of missing data
    real*8,parameter   ::    f=0.5      !scale factor
    real*8,parameter   ::    epsilon=1d-6
    
    contains
        subroutine ReadFile
        implicit none
        integer*4    ::    i
        integer*4    ::    alive
        integer*4    ::    error
        character*100::    filename
    
        filename='ssc.txt'
    
        inquire(file=trim(filename), exist=alive)
        if (alive==0) then
            write(*,'(A12)') 'File does not exist'
            stop
        endif

        open(10, file=trim(filename), status='old', iostat=error)
        if (error/=0) then
            write(*,'(A14)') 'File Open Error'
            stop
        endif
    
        allocate(ts(n),x(n))
        allocate(lam(m),c(m))
        allocate(a(m,m),t(m,m))
        allocate(u(m,m),vt(m,m))
        allocate(xk(m,n-m+1),ro(m,m))
        allocate(rc(m,n),rx(n))
    
        do i=1,n
            read(10,*) ts(i)
        enddo
    
        end subroutine
        
        subroutine Creation
        implicit none
        integer*4::    i,k
        integer*4::    num,level
        real*8   ::    x0
    
        num=0
        level=floor(n*sf)
        do while(.true.)
            call RANDOM_SEED()
            call RANDOM_NUMBER(x0)
            k=floor(1+(n-1)*x0)
            if (.not. isnan(ts(k))) then
                num=num+1
                ts(k)=dsqrt(-1d0)
            endif
            if (level==num) exit
        enddo
    
        end subroutine
        
        subroutine Transform
        implicit none
        integer*4::    i
        integer*4::    num
    
        num=0
        xstd=0d0
        xmean=0d0
    
        do i=1,n
            if (.not. isnan(ts(i))) then
                num=num+1
                xmean=xmean+ts(i)
            endif
        enddo
    
        if (num>1) then
            xmean=xmean/num
            do i=1,n
                if (.not. isnan(ts(i))) then
                    xstd=xstd+(ts(i)-xmean)**2d0
                endif
            enddo
            xstd=dsqrt(xstd/(num-1d0))
            do i=1,n
                x(i)=(ts(i)-xmean)/xstd
            enddo
        else
            write(*,'(A12)') 'No Usable Data'
            stop
        endif
    
        end subroutine
        
        subroutine Toeplitz
        implicit none
        integer*4::    i,j
        integer*4::    num,loc
        real*8   ::    prod
    
        do j=1,m
            num=0
            c(j)=0d0
            do i=1,n-j+1
                prod=x(i)*x(i+j-1)
                if (.not. isnan(prod)) then
                    num=num+1
                    c(j)=c(j)+prod
                endif
            enddo
            c(j)=c(j)/(num-1d0)
        enddo
    
        do i=1,m
            do j=1,m
                loc=abs(i-j)
                a(i,j)=c(loc+1)/m
                t(i,j)=a(i,j)
            enddo
        enddo

        end subroutine
        
        subroutine Decomposition
        implicit none
        integer*4::    i,j
        integer*4::    ii,jj
        integer*4::    num
        integer*4::    info
        real*8   ::    d(m),e(m-1),tau(m)
        real*8   ::    work(m),cwork(2*m),lwork(5*m)
        real*8   ::    C1(m,m),C2(m,m),C3(m,m),C4(m,m)
        real*8   ::    C5(m,m),C6(m,m),C7(m,m),C8(m,m)
        real*8   ::    C9(m),G(m,m),y(m)
    
    
        call dsytrd('U',m,a,m,d,e,tau,work,m,info)
        call dorgtr('U',m,a,m,tau,work,m,info)
        call dsteqr('V',m,d,e,a,m,cwork,info)
    
        do i=1,m
            lam(i)=d(m-i+1)
            ro(:,i)=a(:,m-i+1)
        enddo

        do i=1,n-m+1   
            num=0
            do j=1,m
                if (isnan(x(i+j-1))) then
                    num=num+1
                endif
            enddo
            if (num>m*f) then
                cycle
            endif
        
            do ii=1,m
                do jj=1,m
                    if (ii==jj) then
                        G(ii,jj)=1d0
                    else
                        G(ii,jj)=0d0
                    endif
                    do j=1,m
                        if (isnan(x(i+j-1))) then
                            G(ii,jj)=G(ii,jj)-ro(j,ii)*ro(j,jj)
                        endif
                    enddo
                enddo
            enddo
        
            do ii=1,m
                y(ii)=0d0
                do j=1,m
                   if (.not. isnan(x(i+j-1))) then
                       y(ii)=y(ii)+x(i+j-1)*ro(j,ii)
                   endif
                enddo
            enddo
        
            do ii=1,m
                do jj=1,m
                    if (ii/=jj) then
                        C1(ii,jj)=0d0
                    else
                        C1(ii,jj)=lam(ii)
                    endif
                enddo
            enddo
        
            call dgemm('T','N',m,m,m,1d0,G,m,C1,m,0d0,C2,m)    !C2=GT*C1
            call dgemm('N','N',m,m,m,1d0,C2,m,G,m,0d0,C3,m)    !C3=GT*C1*G
            call dgesvd('A','A',m,m,C3,m,d,u,m,vt,m,lwork,5*m,info)    !C3=U*d*VT
        
            do ii=1,m
                if (dabs(d(ii))>epsilon) then
                    d(ii)=1d0/d(ii)
                endif
            enddo
        
            
            do ii=1,m
                do jj=1,m
                    if (ii/=jj) then
                        C4(ii,jj)=0d0
                    else
                        C4(ii,jj)=d(ii)
                    endif
                enddo
            enddo
        
            call dgemm('T','N',m,m,m,1d0,vt,m,C4,m,0d0,C5,m)    !C5=V*C4
            call dgemm('N','T',m,m,m,1d0,C5,m,u,m,0d0,C6,m)     !C6=V*C4*UT
            call dgemm('N','T',m,m,m,1d0,C1,m,G,m,0d0,C7,m)     !C7=C1*GT
            call dgemm('N','N',m,m,m,1d0,C7,m,C6,m,0d0,C8,m)    !C8=C7*C6
            call dgemm('N','N',m,1,m,1d0,C8,m,y,m,0d0,C9,m)     !C9=C8*y
        
            do j=1,m
                xk(j,i)=C9(j)
            enddo
        enddo
    
        end subroutine
        
        subroutine Reconstructed
        implicit none
        integer*4::    i,j,k
        integer*4::    num
        real*8   ::    prod
    
        do k=1,m
            do i=1,n
                num=0
                rc(k,i)=0d0
                if (i<m) then
                    do j=1,i
                        prod=xk(k,i-j+1)*ro(j,k)
                        if (.not. isnan(prod)) then
                            num=num+1
                            rc(k,i)=rc(k,i)+prod
                        endif
                    enddo
                    rc(k,i)=rc(k,i)/num                    
                else if (i<n-m+1) then
                    do j=1,m
                        prod=xk(k,i-j+1)*ro(j,k)
                        if (.not. isnan(prod)) then
                            num=num+1
                            rc(k,i)=rc(k,i)+prod
                        endif
                    enddo
                    rc(k,i)=rc(k,i)/num                 
                else if (i<=n) then
                    do j=i-n+m,m
                        prod=xk(k,i-j+1)*ro(j,k)
                        if (.not. isnan(prod)) then
                            num=num+1
                            rc(k,i)=rc(k,i)+prod
                        endif
                    enddo
                    rc(k,i)=rc(k,i)/num              
                endif
            enddo
        enddo
    
        do i=1,n
            rx(i)=0d0
            do j=1,npc
                if (rc(j,i)/=0d0) then
                    rx(i)=rx(i)+rc(j,i)
                endif
            enddo
            if (rx(i)/=0d0) then
                rx(i)=rx(i)*xstd+xmean
            else
                rx(i)=dsqrt(-1d0)
            endif
        enddo
    
        end subroutine

        subroutine OutputFile
        implicit none
        integer*4    ::    i,j
        integer*4    ::    error
        character*100::    filename
    
        filename='TimeSeris.txt'
        open(10, file=trim(filename), status='replace', iostat=error)
        if (error/=0) then
            write(*,'(A14)') 'File Open Error'
            stop
        endif
        do i=1,m
            write(10,'(F20.6)') ts(i)
        enddo
        close(10)
    
        filename='ReconstructComponent.txt'
        open(10, file=trim(filename), status='replace', iostat=error)
        if (error/=0) then
            write(*,'(A14)') 'File Open Error'
            stop
        endif
        do i=1,m
            do j=1,n
                write(10,'(F20.6\)') rc(i,j)
            enddo
            write(10,*)
        enddo
        close(10)
    
        filename='ReconstructSeries.txt'
        open(10, file=trim(filename), status='replace', iostat=error)
        if (error/=0) then
            write(*,'(A14)') 'File Open Error'
            stop
        endif
        do i=1,n
            write(10,'(F20.6)') rx(i)
        enddo
        close(10)
    
        deallocate(ts,x)
        deallocate(lam,c)
        deallocate(a,t)
        deallocate(u,vt)
        deallocate(xk,ro)
        deallocate(rc,rx)

        end subroutine
    
    end module